//
//  ColorPickerModel.swift
//  ColorPicker
//
//  Created by Akira Hayashi on 2022/09/24.
//

import Foundation
import SwiftUI

class ColorPickerModel : ObservableObject {
    /// 現在値: Red
    @Published var red: Double = 0.0
    
    /// 現在値: Green
    @Published var green: Double = 0.0
    
    /// 現在値: Blue
    @Published var blue: Double = 0.0
    
    /// 選択色
    var color: Color {
        Color(red: red, green: green, blue: blue)
    }
    
    /// チャネル
    enum Channel {
        case red
        case green
        case blue
    }
    
    /// 現在値を変更する
    /// - Parameters:
    ///   - value: 新しい値。範囲外のときは丸め込みされる
    ///   - channel: 変更するチャネル
    func changeCurrentValue(_ value: Double, channel: Channel) {
        var newValue = value
        if newValue < 0.0 {
            newValue = 0.0
        } else if newValue > 1.0 {
            newValue = 1.0
        }
        
        switch channel {
        case .red:
            red = newValue
        case .green:
            green = newValue
        case .blue:
            blue = newValue
        }
    }
    
    /// 入力中のテキスト: Red
    private var fieldTextStoreRed: String?
    
    /// 入力中のテキスト: Green
    private var fieldTextStoreGreen: String?
    
    /// 入力中のテキスト: Blue
    private var fieldTextStoreBlue: String?

    /// 入力中のテキストのバインディング: Red
    var fieldTextRed: Binding<String> {
        Binding {
            self.fieldTextStoreRed ?? String(format: "%g", self.red)
        } set: {
            self.fieldTextStoreRed = $0
            if let value = Double($0) {
                self.changeCurrentValue(value, channel: .red)
            }
        }
    }
    
    /// 入力中のテキストのバインディング: Green
    var fieldTextGreen: Binding<String> {
        Binding {
            self.fieldTextStoreGreen ?? String(format: "%g", self.green)
        } set: {
            self.fieldTextStoreGreen = $0
            if let value = Double($0) {
                self.changeCurrentValue(value, channel: .green)
            }
        }
    }
    
    /// 入力中のテキストのバインディング: Blue
    var fieldTextBlue: Binding<String> {
        Binding {
            self.fieldTextStoreBlue ?? String(format: "%g", self.blue)
        } set: {
            self.fieldTextStoreBlue = $0
            if let value = Double($0) {
                self.changeCurrentValue(value, channel: .blue)
            }
        }
    }
    
    /// 入力中のテキストをクリアする
    /// - Parameters:
    ///   - channel: チャネル
    func clearFieldText(channel: Channel) {
        switch channel {
        case .red:
            fieldTextStoreRed = nil
        case .green:
            fieldTextStoreGreen = nil
        case .blue:
            fieldTextStoreBlue = nil
        }
    }
    
    /// グラデーションビューのサイズ: Red
    @Published var gradationViewSizeRed: CGSize = CGSize()
    
    /// グラデーションビューのサイズ: Green
    @Published var gradationViewSizeGreen: CGSize = CGSize()
    
    /// グラデーションビューのサイズ: Blue
    @Published var gradationViewSizeBlue: CGSize = CGSize()
    
    /// グラデーションの開始色: Red
    var startColorRed: Color {
        Color(red: 0.0, green: self.green, blue: self.blue)
    }
    
    /// グラデーションの終了色: Red
    var endColorRed: Color {
        Color(red: 1.0, green: self.green, blue: self.blue)
    }
    
    /// グラデーションの開始色: Green
    var startColorGreen: Color {
        Color(red: self.red, green: 0.0, blue: self.blue)
    }
    
    /// グラデーションの終了色: Green
    var endColorGreen: Color {
        Color(red: self.red, green: 1.0, blue: self.blue)
    }
    
    /// グラデーションの開始色: Blue
    var startColorBlue: Color {
        Color(red: self.red, green: self.green, blue: 0.0)
    }
    
    /// グラデーションの終了色: Blue
    var endColorBlue: Color {
        Color(red: self.red, green: self.green, blue: 1.0)
    }
}
