//
//  PickerGradationView.swift
//  ColorPicker
//
//  Created by Akira Hayashi on 2022/09/02.
//

import SwiftUI

struct PickerGradationView: View {

    var channel: ColorPickerModel.Channel
    @ObservedObject var model: ColorPickerModel
    
    /// ノブの大きさ
    let indicatorSize: CGFloat = 20
    /// グラデーションビューの高さ
    let gradationHeigh: CGFloat = 100
    
    /// ノブのX座標
    var indicatorX: CGFloat {
        switch channel {
        case .red:
            return indicatorXRed
        case .green:
            return indicatorXGreen
        case .blue:
            return indicatorXBlue
        }
    }
    
    var indicatorXRed: CGFloat {
        model.gradationViewSizeRed.width * model.red - indicatorSize / 2
    }
    
    var indicatorXGreen: CGFloat {
        model.gradationViewSizeGreen.width * model.green - indicatorSize / 2
    }
    
    var indicatorXBlue: CGFloat {
        model.gradationViewSizeBlue.width * model.blue - indicatorSize / 2
    }
    
    var drag: some Gesture {
        DragGesture()
            .onChanged { dragValue in
                let width: Double
                switch channel {
                case .red:
                    width = self.model.gradationViewSizeRed.width
                case .green:
                    width = self.model.gradationViewSizeGreen.width
                case .blue:
                    width = self.model.gradationViewSizeBlue.width
                }
                
                let value = dragValue.location.x / width
                
                model.changeCurrentValue(value, channel: channel)
                model.clearFieldText(channel: channel)
            }
    }
    
    var startColor: Color {
        switch channel {
        case .red:
            return model.startColorRed
        case .green:
            return model.startColorGreen
        case .blue:
            return model.startColorBlue
        }
    }
    
    var endColor: Color {
        switch channel {
        case .red:
            return model.endColorRed
        case .green:
            return model.endColorGreen
        case .blue:
            return model.endColorBlue
        }
    }
    
    var body: some View {
        ZStack(alignment: .leading) {
            GeometryReader() { geometry in
                LinearGradient(gradient: Gradient(colors: [startColor, endColor]),
                               startPoint: UnitPoint(x: 0, y: 0),
                               endPoint: UnitPoint(x: 1, y: 0))
            }
            .frame(height: gradationHeigh)
            .background {
                GeometryReader { geometry in
                    Path { path in
                        Task.detached {
                            await updateViewSize(geometry.size)
                        }
                    }
                }
            }
            .onTapGesture { point in
                switch self.channel {
                case .red:
                    model.red = point.x / model.gradationViewSizeRed.width
                case .green:
                    model.green = point.x / model.gradationViewSizeGreen.width
                case .blue:
                    model.blue = point.x / model.gradationViewSizeBlue.width
                }
                model.clearFieldText(channel: channel)
            }
            .gesture(drag)

            PickerIndicator()
                .frame(width: indicatorSize, height: indicatorSize)
                .offset(x: indicatorX)
        }
    }
    
    @MainActor
    func updateViewSize(_ size: CGSize) async {
        switch channel {
        case .red:
            if size != model.gradationViewSizeRed {
                model.gradationViewSizeRed = size
            }
            
        case .green:
            if size != model.gradationViewSizeGreen {
                model.gradationViewSizeGreen = size
            }
            
        case .blue:
            if size != model.gradationViewSizeBlue {
                model.gradationViewSizeBlue = size
            }
        }
    }
}

struct PickerGradationView_Previews: PreviewProvider {
    static var previews: some View {
        PickerGradationView(channel: .red,
                            model: ColorPickerModel())
            .padding()
    }
}
